unit Data;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, Db;

type
  TmodData = class(TDataModule)
    OraDB: TDatabase;
    tbPrototype: TTable;
    Query1: TQuery;
    DataSource1: TDataSource;
    Query2: TQuery;
  private
  protected
    function IsValidPlanTable(const APlanTableName: string): Boolean;
  public
    PlanTable: string;
    procedure Connect(const ADatabase, AUser, APassword: string);
    procedure Disconnect;
    function Connected: Boolean;
    procedure NewPlanTable(const APlanTableName: string);
    procedure GetValidPlanTables(AList: TStrings);
    function ExecuteQuery(AList: TStrings): Boolean;
    procedure AnalyseQuery(AList: TStrings);
  end;

var
  modData: TmodData;

implementation

{$R *.DFM}

const
  SCreatePlanTable =
    'create table %s ('#13#10 +
    '  statement_id    varchar2(30),'#13#10 +
    '  timestamp       date,'#13#10 +
    '  remarks         varchar2(80),'#13#10 +
    '  operation       varchar2(30),'#13#10 +
    '  options         varchar2(30),'#13#10 +
    '  object_node     varchar2(30),'#13#10 +
    '  object_owner    varchar2(30),'#13#10 +
    '  object_name     varchar2(30),'#13#10 +
    '  object_instance number,'#13#10 +
    '  object_type     varchar2(30),'#13#10 +
    '  search_columns  number,'#13#10 +
    '  id              number,'#13#10 +
    '  parent_id       number,'#13#10 +
    '  position        number,'#13#10 +
    '  other           long)';
  SDeletePlan =
    'delete from %s where statement_id = ''oraplan''';
  SExplainPlan =
    'select level, id, parent_id, operation, options, object_name, position'#13#10 +
    'from %s'#13#10 +
    'start with id = 0 and statement_id = ''oraplan'''#13#10 +
    'connect by prior id = parent_id and statement_id = ''oraplan''';

{ Auxiliary functions }

function EqualDefs(FD1, FD2: TFieldDefs): Boolean;
var
  I, J: Integer;
begin
  Result := False;
  if FD1.Count <> FD2.Count then Exit;
  for I := 0 to FD1.Count - 1 do
  begin
    J := FD2.IndexOf(FD1[I].Name);
    if (J = -1) or (FD1[I].DataType <> FD2[J].DataType)
      or (FD1[I].Size <> FD2[J].Size) then
      Exit;
  end;
  Result := True;
end;

{ TmodData }

procedure TmodData.AnalyseQuery(AList: TStrings);
begin
  Query2.Close;
  Query2.DatabaseName := OraDB.DatabaseName;
  Query2.SQL.Text := Format(SDeletePlan, [PlanTable]);
  Query2.ExecSQL;
  Query2.SQL := AList;
  Query2.SQL.Insert(0, 'explain plan set statement_id = ''oraplan'' into ' +
    PlanTable + ' for');
  Query2.ExecSQL;
  Query2.SQL.Text := Format(SExplainPlan, [PlanTable]);
  Query2.Open;
end;

procedure TmodData.Connect(const ADatabase, AUser, APassword: string);
begin
  Disconnect;
  try
    OraDB.Params.Values['SERVER NAME'] := ADatabase;
    OraDB.Params.Values['USER NAME'] := AUser;
    OraDB.Params.Values['PASSWORD'] := APassword;
  finally
    OraDB.Open;
  end;
end;

function TmodData.Connected: Boolean;
begin
  Result := OraDB.Connected;
end;

procedure TmodData.Disconnect;
begin
  OraDB.Close;
end;

function TmodData.ExecuteQuery(AList: TStrings): Boolean;
begin
  Query1.Close;
  Query1.DatabaseName := OraDB.DatabaseName;
  Query1.SQL := AList;
  Result := True;
  try
    Query1.Open;
  except
    on E: ENoResultSet do
      Result := False;
  end;
end;

procedure TmodData.GetValidPlanTables(AList: TStrings);
var
  I: Integer;
begin
  ASSERT(Connected);
  AList.BeginUpdate;
  try
    Session.GetTableNames(OraDB.DatabaseName, '', False, False, AList);
    for I := AList.Count - 1 downto 0 do
      if not IsValidPlanTable(AList[I]) then
        AList.Delete(I);
  finally
    AList.EndUpdate;
  end;
end;

function TmodData.IsValidPlanTable(const APlanTableName: string): Boolean;
begin
  with TTable.Create(nil) do
  try
    DatabaseName := OraDB.DatabaseName;
    TableName := APlanTableName;
    FieldDefs.Update;
    Result := EqualDefs(FieldDefs, tbPrototype.FieldDefs);
    Free;
  except
    Result := False;
    Free;
  end;
end;

procedure TmodData.NewPlanTable(const APlanTableName: string);
begin
  ASSERT(Connected, 'No current database');
  with TQuery.Create(nil) do
  try
    DatabaseName := OraDB.DatabaseName;
    SQL.Text := Format(SCreatePlanTable, [APlanTableName]);
    ExecSQL;
  finally
    Free;
  end;
  PlanTable := APlanTableName;
end;

end.
